package kz.project.printedFormsService.config;

import kz.project.printedFormsService.exception.AccessDeniedException;
import lombok.experimental.UtilityClass;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.oauth2.jwt.Jwt;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@UtilityClass
public class SecurityContextUtils {
    private static final String PROJECT_ROLE_SPLITTER = "_";

    public static Set<String> getRoles() {
        return SecurityContextHolder
                .getContext()
                .getAuthentication()
                .getAuthorities()
                .stream()
                .map(GrantedAuthority::getAuthority)
                .collect(Collectors.toSet());

    }

    public static String getUsername() {
        return SecurityContextHolder
                .getContext()
                .getAuthentication()
                .getName();
    }

    public static Map<String, List<String>> getProjectRoleMap() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();

        if (authentication != null && authentication.getPrincipal() instanceof Jwt jwt) {

            // Access resource_access claims
            Map<String, Object> resourceAccess = jwt.getClaim("resource_access");

            // Extract roles or permissions from resource_access
            if (resourceAccess != null && resourceAccess.containsKey("print_form")) {
                Map<String, Object> yourResourceAccess = (Map<String, Object>) resourceAccess.get("print_form");

                // Extract roles from yourResourceAccess
                if (yourResourceAccess != null && yourResourceAccess.containsKey("roles")) {
                    List<String> roles = (List<String>) yourResourceAccess.get("roles");
                    return roles.stream().collect(Collectors.toMap(
                                    role -> role.split(PROJECT_ROLE_SPLITTER)[0],
                                    role -> new ArrayList<>(List.of(role.split(PROJECT_ROLE_SPLITTER)[1])),
                                    (existing, addition) -> {
                                        existing.addAll(addition);
                                        return existing;
                                    }
                            )
                    );
                }
            }
        }
        return null;
    }

    public static String getGitlabProjectBranchFromRole() {

        return SecurityContextHolder
                .getContext()
                .getAuthentication()
                .getAuthorities()
                .stream()
                .map(GrantedAuthority::getAuthority)
                .filter(role -> role.contains("gitlab"))
                .map(role -> role.replace("ROLE_gitlab_", ""))
                .findFirst()
                .orElseThrow(() -> new AccessDeniedException("Не указана роль для доступа к gitlab"));
    }
}
