package kz.project.printedFormsService.data.dto;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import kz.project.printedFormsService.data.entity.TemplateEntity;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@Schema(name = "TemplateShortDto", description = "Модель")
public class TemplateShortDto {
    @NotBlank
    @Schema(name ="templateId", description = "Код шаблона", defaultValue = "123")
    private Long templateId;

    @NotBlank
    @Schema(name ="code", description = "Код шаблона", defaultValue = "template_1")
    private String code;
    @NotBlank
    @Schema(name ="name", description = "Наименование шаблона",defaultValue = "Шаблон 1")
    private String name;
    @NotBlank
    @Schema(name = "type", description = "Тип фаила")
    private String type;
    @NotBlank
    @Schema(name ="status", description = "Флаг активности", defaultValue = "1")
    private Boolean status;
    @NotBlank
    @Schema(name ="project", description = "Проект")
    private String project;
    @NotBlank
    @Schema(name ="version", description = "Версия активности")
    private Integer version;

    @NotBlank
    @Schema(name ="rights", description = "Права в проекте")
    private List<String> rights;

    public static TemplateShortDto toDto(TemplateEntity te, List<String> projectRoleMap) {
        return TemplateShortDto.builder()
                .templateId(te.getId())
                .name(te.getName())
                .code(te.getCode())
                .status(te.getStatus())
                .project(te.getProject())
                .type(te.getType().getCode())
                .version(te.getVersion())
                .rights(projectRoleMap)
                .build();
    }

    public static Page<TemplateShortDto> toDtoList(Page<TemplateEntity> all, Map<String, List<String>> projectRoleMap) {
        List<TemplateShortDto> dtos = new ArrayList<>();
        if (all != null) {
            for (TemplateEntity te : all.getContent()) {
                dtos.add(TemplateShortDto.toDto(te, projectRoleMap.get(te.getProject())));
            }
            all.getTotalElements();
        }
        return new PageImpl<>(dtos, all.getPageable(), all.getTotalElements());
    }
}
