package kz.project.printedFormsService.service.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import kz.project.printedFormsService.data.dto.DocumentByDateDto;
import kz.project.printedFormsService.data.dto.DocumentByDateDto.DocumentCount;
import kz.project.printedFormsService.data.dto.DocumentByTemplateDto;
import kz.project.printedFormsService.data.entity.DocumentEntity;
import kz.project.printedFormsService.data.entity.TemplateEntity;
import kz.project.printedFormsService.data.repository.DocumentJdbcRepository;
import kz.project.printedFormsService.data.repository.DocumentRepository;
import kz.project.printedFormsService.data.repository.TemplateRepository;
import kz.project.printedFormsService.exception.ValidationException;
import kz.project.printedFormsService.service.DocumentsService;
import kz.project.printedFormsService.service.TemplateService;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Map;
import java.util.Set;

@Service
@RequiredArgsConstructor
public class DocumentsServiceImpl implements DocumentsService {

    private final DocumentJdbcRepository documentJdbcRepository;
    private final TemplateService templateService;
    private final DocumentRepository documentRepository;

    @Override
    public @NotNull DocumentByDateDto getDocumentCountsByDate(@NotNull LocalDate startDate, @NotNull LocalDate endDate, @NotNull Set<String> projects) {
        Map<LocalDate, Integer> documentsByDate = documentJdbcRepository.getDocumentCountsByDay(startDate, endDate, projects);
        return new DocumentByDateDto("success", "Данные успешно получены")
                .setData(DocumentByDateDto.DocumentsData.builder()
                        .startDate(startDate)
                        .endDate(endDate)
                        .documentCounts(
                                documentsByDate.entrySet()
                                        .stream()
                                        .map(entry -> DocumentCount.builder()
                                                .date(entry.getKey())
                                                .count(entry.getValue())
                                                .build()

                                        )
                                        .toList()
                        )
                        .build()
                );
    }

    @Override
    public @NotNull DocumentByTemplateDto getDocumentCountsByTemplate(@NotNull LocalDate startDate, @NotNull LocalDate endDate, @NotNull Set<String> projects) {
        return new DocumentByTemplateDto("success", "Данные успешно получены")
                .setData(DocumentByTemplateDto.DocumentsData.builder()
                        .startDate(startDate)
                        .endDate(endDate)
                        .templateCounts(documentJdbcRepository.getDocumentsCountByTemplate(startDate, endDate, projects)
                                .stream()
                                .map(documentCountByTemplate -> DocumentByTemplateDto.TemplateCount.builder()
                                        .templateId(documentCountByTemplate.getTemplateId())
                                        .templateName(documentCountByTemplate.getTemplateName())
                                        .totalCount(documentCountByTemplate.getCount())
                                        .build()
                                )
                                .toList()
                        )
                        .build()
                );
    }
 @Transactional
    @Override
    public void saveDocument(Long templateId) throws ValidationException, JsonProcessingException {
      DocumentEntity documentEntity = new DocumentEntity();
        TemplateEntity templateEntity = templateService.getTemplateEntity(templateId);
        documentEntity.setTemplate(templateEntity);
        documentEntity.setProject(templateEntity.getProject());
        documentEntity.setCreatedAt(LocalDateTime.now());
        documentRepository.save(documentEntity);


    }
}
