package kz.project.printedFormsService.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import kz.project.printedFormsService.config.SecurityContextUtils;
import kz.project.printedFormsService.data.dto.GitlabUploadRequest;
import kz.project.printedFormsService.data.dto.TemplateDto;
import kz.project.printedFormsService.data.dto.TemplateResponseDto;
import kz.project.printedFormsService.service.GitlabClient;
import kz.project.printedFormsService.service.TemplateService;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

@Service
@RequiredArgsConstructor
public class GitlabTemplateServiceImpl implements kz.project.printedFormsService.service.GitlabTemplateService {

    private final GitlabClient gitlabClient;

    private final TemplateService templateService;

    private final ObjectMapper objectMapper;


    @SneakyThrows
    @Override
    public TemplateDto saveFromGitlab(GitlabUploadRequest gitlabUploadRequest) {
        String gitlabBranch = SecurityContextUtils.getGitlabProjectBranchFromRole();
        List<MultipartFile> filesToSave = new ArrayList<>();

        filesToSave.add(gitlabClient.downloadFileAsMultipartFile(
                gitlabUploadRequest.getRepository(),
                gitlabUploadRequest.getFileDataPath(),
                gitlabBranch
        ));

        String dtoString = gitlabClient.downloadFileContentAsString(
                        gitlabUploadRequest.getRepository(),
                        gitlabUploadRequest.getFileDtoPath(),
                        gitlabBranch
                )
                .blockOptional()
                .orElseThrow();

        TemplateDto templateDto = objectMapper.readValue(dtoString, TemplateDto.class);

        if (gitlabUploadRequest.getFileHeaderPath() != null) {
            filesToSave.add(
                    gitlabClient.downloadFileAsMultipartFile(
                            gitlabUploadRequest.getRepository(),
                            gitlabUploadRequest.getFileHeaderPath(),
                            gitlabBranch
                    )
            );
        }

        return templateService.save(templateDto, filesToSave);
    }

    @SneakyThrows
    @Override
    public TemplateResponseDto updateFromGitlab(GitlabUploadRequest gitlabUploadRequest) {
        String gitlabBranch = SecurityContextUtils.getGitlabProjectBranchFromRole();
        List<MultipartFile> filesToSave = new ArrayList<>();

        filesToSave.add(gitlabClient.downloadFileAsMultipartFile(
                gitlabUploadRequest.getRepository(),
                gitlabUploadRequest.getFileDataPath(),
                gitlabBranch
        ));

        String dtoString = gitlabClient.downloadFileContentAsString(
                        gitlabUploadRequest.getRepository(),
                        gitlabUploadRequest.getFileDtoPath(),
                        gitlabBranch
                )
                .blockOptional()
                .orElseThrow();

        TemplateDto templateDto = objectMapper.readValue(dtoString, TemplateDto.class);

        if (gitlabUploadRequest.getFileHeaderPath() != null) {
            filesToSave.add(
                    gitlabClient.downloadFileAsMultipartFile(
                            gitlabUploadRequest.getRepository(),
                            gitlabUploadRequest.getFileHeaderPath(),
                            gitlabBranch
                    )
            );
        }

        return templateService.edit(templateDto, filesToSave);
    }

}
